%bcond metis 1
%bcond netcdf 1
%bcond openssl 1
%bcond parmetis 1
%bcond check 1
%bcond mpich 1
%bcond openmpi 1
%bcond static 0

Name:           scales-ppm
Version:        1.0.8.2
Release:        1%{?dist}
Summary:        ScalES-PPM: Scalable Earth System Models Partitioning and Parallelization Modules

License:        BSD
URL:            https://swprojects.dkrz.de/redmine/projects/%{name}
Source0:        https://swprojects.dkrz.de/redmine/attachments/download/526/ppm-%{version}.tar.xz

BuildRequires:  doxygen
BuildRequires:  gcc-gfortran
BuildRequires:  perl(Data::Dumper)
BuildRequires:  perl(File::Spec)
BuildRequires:  perl(File::Temp)
BuildRequires:  perl(Getopt::Long)
BuildRequires:  perl(IPC::Open2)
BuildRequires:  perl(Pod::Usage)
BuildRequires:  perl(Storable)
BuildRequires:  texlive-dvips
BuildRequires:	texlive-dvips-bin
%if %{with metis}
BuildRequires:  metis-devel
%endif
%if %{with netcdf}
BuildRequires:  netcdf-fortran-devel
%endif
%if %{with openssl}
BuildRequires:  openssl-devel
%endif
%if %{with mpich}
BuildRequires:  mpich-devel
%if %{with netcdf}
BuildRequires:  netcdf-fortran-mpich-devel
%endif
%if %{with parmetis}
BuildRequires:  ptscotch-mpich-devel-parmetis
%endif
%endif
%if %{with openmpi}
BuildRequires:  openmpi-devel
%if %{with netcdf}
BuildRequires:  netcdf-fortran-openmpi-devel
%endif
%if %{with parmetis}
BuildRequires:  ptscotch-openmpi-devel-parmetis
%endif
%endif

%if %{with mpich}
%global mpi_list mpich
%endif
%if %{with openmpi}
%global mpi_list %{?mpi_list} openmpi
%endif

%description
Convenience library to provide parallelization and performance oriented modules
to parallel software packages, particularly for earth system models.

Functionality includes:

  - parallel Linear Algebra Solver
  - partitioning routines
  - convenient interfaces to commonly used data-structures and APIs

This package contains libraries for %{name}.

%package        doc
Summary:        Documentation files for %{name}
BuildArch:      noarch

%description    doc
This package contains documentation files for developing applications that use
%{name}.

%package        devel
Summary:        Development files for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description    devel
The %{name}-devel package contains libraries and header files for developing
applications that use %{name}.

%if %{with static}
%package        static
Summary:        Static libraries for %{name}
Requires:       %{name}-devel%{?_isa} = %{version}-%{release}

%description    static
The %{name}-static package contains static libraries for developing applications
that use %{name}.
%endif

%if %{with mpich}
%package        mpich
Summary:        MPICH MPI libraries for %{name}

%description    mpich
This package contains MPICH MPI libraries for %{name}.

%package        mpich-devel
Summary:        Development files for %{name}-mpich
Requires:       %{name}-mpich%{?_isa} = %{version}-%{release}

%description    mpich-devel
The %{name}-mpich-devel package contains libraries and header files for
developing applications that use %{name}-mpich.

%if %{with static}
%package        mpich-static
Summary:        Static libraries for %{name}-mpich
Requires:       %{name}-mpich-devel%{?_isa} = %{version}-%{release}

%description    mpich-static
The %{name}-mpich-static package contains static libraries for developing
applications that use %{name}-mpich.
%endif
%endif

%if %{with openmpi}
%package        openmpi
Summary:        OpenMPI MPI libraries for %{name}

%description    openmpi
This package contains OpenMPI MPI libraries for %{name}.

%package        openmpi-devel
Summary:        Development files for %{name}-openmpi
Requires:       %{name}-openmpi%{?_isa} = %{version}-%{release}

%description    openmpi-devel
The %{name}-openmpi-devel package contains libraries and header files for
developing applications that use %{name}-openmpi.

%if %{with static}
%package        openmpi-static
Summary:        Static libraries for %{name}-openmpi
Requires:       %{name}-openmpi-devel%{?_isa} = %{version}-%{release}

%description    openmpi-static
The %{name}-openmpi-static package contains static libraries for developing
applications that use %{name}-openmpi.
%endif
%endif

%prep
%autosetup -n ppm-%{version} -p1

%build
# Common configure options
# Dependency tracking necessary for f90 module dependencies
%global configure_opts \\\
  --enable-dependency-tracking \\\
  %{!?with_static:--disable-static} \\\
  %{!?with_metis:--disable-metis} \\\
  %{!?with_netcdf:--disable-netcdf} \\\
  %{!?with_openssl:--disable-crypto}

mkdir %{_vpath_builddir}
cd %{_vpath_builddir}
ln -s ../configure .
%configure %{configure_opts} \
  --disable-MPI
%make_build
# Documentation
doxygen -u doc/unitdoc/scales-ppm.doxygen
%make_build -C doc/unitdoc html-local
cd -

# Workaround for building against MPICH with GCC-10
# https://gcc.gnu.org/gcc-10/porting_to.html#argument-mismatch
%global build_fflags %{build_fflags} -fallow-argument-mismatch
unset FCFLAGS
unset FFLAGS

# Specify additional MPI_LAUNCH args for tests
MPICH_MPI_LAUNCH_ARGS="-iface lo"
OPENMPI_MPI_LAUNCH_ARGS="--oversubscribe"

# MPI builds
for mpi in %{mpi_list}; do
  mkdir ${mpi}-%{_vpath_builddir}
  cd ${mpi}-%{_vpath_builddir}
  module load mpi/${mpi}-%{_arch}
  ln -s ../configure .
  %configure %{configure_opts} \
    FCFLAGS="$FCFLAGS -I${MPI_FORTRAN_MOD_DIR}" \
    MPI_LAUNCH="/usr/lib64/${mpi}/bin/mpirun $(eval echo -e \"\${${mpi^^}_MPI_LAUNCH_ARGS}\")" \
    CC=mpicc \
    FC=mpifort \
    %{!?with_parmetis:--disable-parmetis} \
    --libdir=%{_libdir}/${mpi}/lib \
    --bindir=%{_libdir}/${mpi}/bin \
    --sbindir=%{_libdir}/${mpi}/sbin \
    --includedir=%{_includedir}/${mpi}-%{_arch} \
    --datarootdir=%{_libdir}/${mpi}/share \
    --mandir=%{_libdir}/${mpi}/share/man
  %make_build
  module purge
  cd -
done

%install
%make_install -C %{_vpath_builddir}
rm -f ${RPM_BUILD_ROOT}/%{_libdir}/*.la

for mpi in %{mpi_list}; do
  module load mpi/${mpi}-%{_arch}
  %make_install -C ${mpi}-%{_vpath_builddir}
  rm -f ${RPM_BUILD_ROOT}/%{_libdir}/${mpi}/lib/*.la
  module purge
done

%check
%if %{with check}
%make_build -C %{_vpath_builddir} check
for mpi in %{mpi_list}; do
  module load mpi/${mpi}-%{_arch}
  %make_build -j1 -C ${mpi}-%{_vpath_builddir} check
  module purge
done
%endif

%files
%license COPYING
%license LICENSE
%doc AUTHORS
%{_libdir}/*.so.0*

%files          doc
%doc doc/cfortran.*
%doc doc/README.codingconventions
%doc %{_vpath_builddir}/doc/unitdoc/html
%doc example/*.c
%doc example/*.f90

%files          devel
%{_includedir}/*
%{_libdir}/*.so
%{_libdir}/pkgconfig/%{name}.pc
%{_libdir}/pkgconfig/%{name}-core.pc

%if %{with static}
%files          static
%{_libdir}/*.a
%endif

%if %{with mpich}
%files          mpich
%{_libdir}/mpich/lib/*.so.0*

%files          mpich-devel
%{_includedir}/mpich-%{_arch}/*
%{_libdir}/mpich/lib/*.so
%{_libdir}/mpich/lib/pkgconfig/%{name}.pc
%{_libdir}/mpich/lib/pkgconfig/%{name}-core.pc

%if %{with static}
%files          mpich-static
%{_libdir}/mpich/lib/*.a
%endif
%endif

%if %{with openmpi}
%files          openmpi
%{_libdir}/openmpi/lib/*.so.0*

%files          openmpi-devel
%{_includedir}/openmpi-%{_arch}/*
%{_libdir}/openmpi/lib/*.so
%{_libdir}/openmpi/lib/pkgconfig/%{name}.pc
%{_libdir}/openmpi/lib/pkgconfig/%{name}-core.pc

%if %{with static}
%files          openmpi-static
%{_libdir}/openmpi/lib/*.a
%endif
%endif

%changelog
* Sun Mar  9 2025 Matthew Krupcale <mkrupcale@gmail.com> - 1.0.8.2-1
- Initial package
